#include "define.h"

#pragma warning(disable:4800) 

using namespace ssw;

ulong File::Read(void *buffer, ulong size)
{
    assert(NULL != buffer);

    if (INVALID_HANDLE_VALUE == m_handle)
        return 0;
    if (0 == size)
        return 0;
    DWORD dwReads;
    ReadFile(m_handle, buffer, size, &dwReads, NULL);
    return (ulong)dwReads;
}

ulong File::Write(void *buffer, ulong size)
{
    assert(NULL != buffer);

    if (INVALID_HANDLE_VALUE == m_handle)
        return 0;
    if (0 == size)
        return 0;
    DWORD dwWrites;
    WriteFile(m_handle, buffer, size, &dwWrites, NULL);
    return (ulong)dwWrites;
}

ulong File::GetSize()
{
    if (INVALID_HANDLE_VALUE == m_handle)
        return 0;
    return m_size;
}

bool File::Seek(ulong offset)
{
    if (INVALID_HANDLE_VALUE == m_handle)
        return 0;
    if (offset < 0)
        return 0;
    LARGE_INTEGER li;
    li.LowPart = offset;
    li.HighPart = 0;
    if (HFILE_ERROR != SetFilePointer(m_handle, li.LowPart, &li.HighPart, FILE_BEGIN))
        return false;
    return true;
}

ulong File::Location()
{
    if (INVALID_HANDLE_VALUE == m_handle)
        return 0;
    return (ulong)SetFilePointer(m_handle, 0, 0, FILE_CURRENT);
}

bool File::Open(const wchar_t *path, OPEN_WAY way, OPEN_DISP disp)
{
    assert(NULL != path);

    if (INVALID_HANDLE_VALUE != m_handle)
        return false;
    DWORD access = 0, open_disp = 0;
    // дȨ
    if (READ == way) access = GENERIC_READ;
    else if (WRITE == way) access = GENERIC_WRITE;
    else if (READ_WRITE == way) access = GENERIC_READ | GENERIC_WRITE;
    // Ȩ
    if (OPEN == disp) open_disp = OPEN_EXISTING;
    else if (CREATE == disp) open_disp = CREATE_NEW;
    else if (UPDATE == disp) open_disp = CREATE_ALWAYS;
    if ((0 == access) || (0 == open_disp))
        return false;
    HANDLE handle = CreateFile(path, access, 0, NULL, open_disp, FILE_ATTRIBUTE_NORMAL, NULL);
    if (INVALID_HANDLE_VALUE == handle) 
        return false;
    m_handle = handle;
    m_size = (ulong)GetFileSize(m_handle, NULL);
    return true;
}

void File::Close()
{
    if (INVALID_HANDLE_VALUE == m_handle)
        return;
    CloseHandle(m_handle);
    m_handle = INVALID_HANDLE_VALUE;
    return;
}

File::File()
{
    m_handle = INVALID_HANDLE_VALUE;
    m_size = 0;
    return;
}

File::~File()
{
    this->Close();
    return;
}
