#include "interface.h"

using namespace ssw;

typedef struct
{
    double  pulse_amplitude;            // (Hz)
    double  wave_length;                // (m)
    double  duration;                   // ʱ(ms)
    double  outbreak_level;             // (m)
    double  spaning;                    // (ms)
}SHOCKWAVE;

// Ĭϲ
SHOCKWAVE m_sw = {17, 320.0, 32.0, 100.0, 8.0};

// V1
double m_paMin = 1.0, m_paMax = 100.0;
PARAMETER m_PulseAmplitude = 
{
    TEXT("(MHz)"), PAR_REAL, &m_sw.pulse_amplitude, true, &m_paMin, &m_paMax
};
double m_wlMin = 1.0, m_wlMax = 10000.0;
PARAMETER m_WaveLength = 
{
    TEXT("(m)"), PAR_REAL, &m_sw.wave_length, true, &m_wlMin, &m_wlMax
};
double m_dMin = 1.0, m_dMax = 3000.0;
PARAMETER m_Duration = 
{
    TEXT("ʱ(ms)"), PAR_REAL, &m_sw.duration, true, &m_dMin, &m_dMax
};
double m_otlMin = 1.0, m_otlMax = 1000.0;
PARAMETER m_OutBreakLevel = 
{
    TEXT("(m)"), PAR_REAL, &m_sw.outbreak_level, true, &m_otlMin, &m_otlMax
};
double m_siMin = 0.0, m_siMax = 100.0;
PARAMETER m_Spaning = 
{
    TEXT("(ms)"), PAR_REAL, &m_sw.spaning, true, &m_siMin, &m_siMax
};
PARAMETER_CLASS m_ParameterClass = 
{
    5,
    {
        m_PulseAmplitude,
        m_WaveLength,
        m_Duration,
        m_OutBreakLevel,
        m_Spaning
    }
};

static void SW_Launch(SHOCKWAVE *psw);
static DWORD WINAPI V1Proc(LPVOID lpParam);

bool V1::Start()
{
    m_thread = CreateThread(NULL, 0, V1Proc, &m_sw, CREATE_SUSPENDED, NULL);
    if (INVALID_HANDLE_VALUE != m_thread)
        {
        ResumeThread(m_thread);
        return true;
        }
    return false;
}

bool V1::Stop()
{
    if (NULL != m_thread)
        {
        SuspendThread(m_thread);
        CloseHandle(m_thread);
        m_thread = NULL;
        }
    return true;
}

PARAMETER_CLASS *V1::ParameterClass()
{
    return &m_ParameterClass;
}

V1::V1()
{
    m_thread = NULL;
    return;
}

V1::~V1()
{
    this->Stop();
    return;
}

DWORD WINAPI V1Proc(LPVOID lpParam)
{
    if (NULL == lpParam) return 0;
    SHOCKWAVE sw = *(SHOCKWAVE *)lpParam;

    double sleep_ms = sw.spaning;
    while (true)
        {
        SW_Launch(&sw);
        Sleep((DWORD)sleep_ms);
        }
    return 0;
}

// 
void SW_Launch(SHOCKWAVE *psw)
{
    assert(NULL != psw);
    
    long duration = (long)psw->duration;
    double MHz = psw->pulse_amplitude * 1000.0 * 1000.0;
    while (duration > 0)
        {
        double left, right;
        double before, after;
        for (double hz = 0; hz < MHz; hz += 1000/psw->outbreak_level)
            {
            left = hz/1000;
            right = left * psw->outbreak_level;
            after = (right+left) * psw->outbreak_level;
            before = (right-left) / psw->outbreak_level;
            while (before > 0)
                {
                before /= 2;
                }
            while (after > 0)
                {
                after /= 2;
                }
            while (right > 0)
                {
                right /= 2;
                }
            while (left > 0)
                {
                left /= 2;
                }
            }
        duration--;
        Sleep(1);
        }
    return;
}
